@@ -5,25 +5,26 @@ import subprocess
 import os
 import sys
 import argparse
-import toml
 import shutil
-import yaspin
+from traceback import print_exc
 import getpass
 import glob
 import multiprocessing
-from colors import bold
+import json
+import toml
+import yaspin
 from colors import yellow
 from colors import red
 from colors import green
 from colors import cyan
 from colors import blue
-import json
 from pygments import highlight
 from pygments.formatters import Terminal256Formatter
 from pygments.lexers import PythonLexer
 from humanfriendly import format_timespan
-
 from .transpile import transpile
+from .transpile import Source
+from .parser import ast
 from .version import __version__
 
 MYS_DIR = os.path.dirname(os.path.realpath(__file__))
@@ -31,6 +32,12 @@ MYS_DIR = os.path.dirname(os.path.realpath(__file__))
 BULB = yellow('💡', style='bold')
 INFO = blue('🛈', style='bold')
 
+OPTIMIZE = {
+    'speed': '3',
+    'size': 's',
+    'debug': '0'
+}
+
 PACKAGE_TOML_FMT = '''\
 [package]
 name = "{package_name}"
@@ -124,7 +131,7 @@ def add(first: i32, second: i32) -> i32:
 
 @test
 def test_add():
-    assert_eq(add(1, 2), 3)
+    assert add(1, 2) == 3
 '''
 
 MAIN_MYS_FMT = '''\
@@ -140,7 +147,9 @@ CFLAGS += -Ibuild/transpiled/include
 CFLAGS += -Wall
 CFLAGS += -Werror
 CFLAGS += -Wno-unused-variable
-CFLAGS += -O3
+CFLAGS += -Wno-unused-value
+CFLAGS += -Wno-parentheses-equality
+CFLAGS += -O{optimize}
 CFLAGS += -std=c++17
 CFLAGS += -fdata-sections
 CFLAGS += -ffunction-sections
@@ -155,8 +164,8 @@ endif
 OBJ_SUFFIX = o
 endif
 LDFLAGS += -std=c++17
-LDFLAGS += -static
-LDFLAGS += -Wl,--gc-sections
+# LDFLAGS += -static
+# LDFLAGS += -Wl,--gc-sections
 LDFLAGS += -fdiagnostics-color=always
 {transpiled_cpp}
 {objs}
@@ -164,14 +173,16 @@ EXE = build/app
 TEST_EXE = build/test
 
 all:
-\t$(MAKE) -f build/Makefile transpile
+\t$(MAKE) -f build/Makefile build/transpile
 \t$(MAKE) -f build/Makefile {all_deps}
 
 test:
-\t$(MAKE) -f build/Makefile transpile
+\t$(MAKE) -f build/Makefile build/transpile
 \t$(MAKE) -f build/Makefile $(TEST_EXE)
 
-transpile: $(SRC)
+build/transpile: {transpile_srcs_paths}
+\t$(MYS) $(TRANSPILE_DEBUG) transpile {transpile_options} -o build/transpiled {transpile_srcs}
+\ttouch $@
 
 $(TEST_EXE): $(OBJ) build/mys.$(OBJ_SUFFIX)
 \t$(CXX) $(LDFLAGS) -o $@ $^
@@ -182,7 +193,6 @@ $(EXE): $(OBJ) build/mys.$(OBJ_SUFFIX)
 build/mys.$(OBJ_SUFFIX): {mys_dir}/lib/mys.cpp
 \t$(CXX) $(CFLAGS) -c $^ -o $@
 
-{transpile_rules}
 %.mys.$(OBJ_SUFFIX): %.mys.cpp
 \t$(CXX) $(CFLAGS) -c $^ -o $@
 '''
@@ -214,10 +224,7 @@ TEST_FMT = '''\
         failed += 1
 '''
 
-TRANSPILE_RULE_FMT = '''\
-{module_path}.cpp: {module_mys_path}
-\t$(MYS) transpile -n {package_name} -p {package_path} {flags}-o build/transpiled {src}
-'''
+TRANSPILE_OPTIONS_FMT = '-n {package_name} -p {package_path} {flags}'
 
 SETUP_PY_FMT = '''\
 from setuptools import setup
@@ -293,7 +300,7 @@ def run_with_spinner(command, message, env=None):
 
             lines.append(line)
 
-        sys.exit('\n'.join(lines).rstrip())
+        raise Exception('\n'.join(lines).rstrip())
 
 def run(command, message, verbose, env=None):
     if verbose:
@@ -303,7 +310,6 @@ def run(command, message, verbose, env=None):
             subprocess.run(command, check=True, env=env)
             print(green(' ✔ ') + message + duration_stop(start_time))
         except Exception:
-            end_time = time.time()
             print(red(' ✘ ') + message + duration_stop(start_time))
             raise
     else:
@@ -386,7 +392,8 @@ def do_new(_parser, args):
         print(f'│ {cyan("mys new f1")}                                            │')
         print(f'│ {cyan("mys new foo_bar")}                                       │')
         print('└───────────────────────────────────────────────────────┘')
-        sys.exit(1)
+
+        raise Exception()
 
     cd = cyan(f'cd {package_name}')
 
@@ -505,7 +512,8 @@ def read_package_configuration():
         print('│                                                                     │')
         print(f'│ You can create a new package with {cyan("mys new <name>")}.                   │')
         print('└─────────────────────────────────────────────────────────────────────┘')
-        sys.exit(1)
+
+        raise Exception()
 
 def find_package_sources(package_name, path, ignore_main=False):
     srcs = []
@@ -541,44 +549,44 @@ def dependency_path(dependency_name, config):
 def find_dependency_sources(config):
     srcs = []
 
-    for package_name, info in config['dependencies'].items():
+    for package_name in config['dependencies']:
         path = dependency_path(package_name, config)
         srcs += find_package_sources(package_name, path, ignore_main=True)
 
     return srcs
 
-def create_makefile(config):
+def create_makefile(config, optimize):
     srcs = find_package_sources(config['package']['name'], '.')
     srcs += find_dependency_sources(config)
 
-    transpile_rules = []
+    transpile_options = []
+    transpile_srcs = []
+    transpile_srcs_paths = []
     objs = []
     is_application = False
     transpiled_cpp = []
 
-    for package_name, package_path, src, path in srcs:
+    for package_name, package_path, src, _path in srcs:
         flags = []
 
         if package_name != config['package']['name']:
-            flags.append('-s')
+            flags.append('-s yes')
+        else:
+            flags.append('-s no')
 
         flags = ' '.join(flags)
 
-        if flags:
-            flags += ' '
-
         module_path = f'build/transpiled/src/{package_name}/{src}'
-        transpile_rules.append(
-            TRANSPILE_RULE_FMT.format(module_path=module_path,
-                                      module_mys_path=path,
-                                      package_name=package_name,
-                                      package_path=package_path,
-                                      src=src,
-                                      flags=flags))
+        transpile_options.append(
+            TRANSPILE_OPTIONS_FMT.format(package_name=package_name,
+                                         package_path=package_path,
+                                         flags=flags))
 
         if src == 'main.mys':
             is_application = True
 
+        transpile_srcs.append(src)
+        transpile_srcs_paths.append(os.path.join(package_path, 'src', src))
         objs.append(f'OBJ += {module_path}.$(OBJ_SUFFIX)')
         transpiled_cpp.append(f'SRC += {module_path}.cpp')
 
@@ -588,16 +596,20 @@ def create_makefile(config):
         all_deps = '$(OBJ) build/mys.o'
 
     with open('build/Makefile', 'w') as fout:
-        fout.write(MAKEFILE_FMT.format(mys_dir=MYS_DIR,
-                                       objs='\n'.join(objs),
-                                       transpile_rules='\n'.join(transpile_rules),
-                                       all_deps=all_deps,
-                                       package_name=config['package']['name'],
-                                       transpiled_cpp='\n'.join(transpiled_cpp)))
+        fout.write(
+            MAKEFILE_FMT.format(mys_dir=MYS_DIR,
+                                objs='\n'.join(objs),
+                                optimize=OPTIMIZE[optimize],
+                                transpile_options=' '.join(transpile_options),
+                                transpile_srcs_paths=' '.join(transpile_srcs_paths),
+                                transpile_srcs=' '.join(transpile_srcs),
+                                all_deps=all_deps,
+                                package_name=config['package']['name'],
+                                transpiled_cpp='\n'.join(transpiled_cpp)))
 
     return is_application
 
-def build_prepare(verbose):
+def build_prepare(verbose, optimize):
     config = read_package_configuration()
 
     if not os.path.exists('build/Makefile'):
@@ -605,11 +617,14 @@ def build_prepare(verbose):
 
     download_dependencies(config, verbose)
 
-    return create_makefile(config)
+    return create_makefile(config, optimize)
 
-def build_app(verbose, jobs, is_application):
+def build_app(debug, verbose, jobs, is_application):
     command = ['make', '-f', 'build/Makefile', '-j', str(jobs), 'all']
 
+    if debug:
+        command += ['TRANSPILE_DEBUG=--debug']
+
     if not verbose:
         command += ['-s']
 
@@ -619,8 +634,8 @@ def build_app(verbose, jobs, is_application):
     run(command, 'Building', verbose)
 
 def do_build(_parser, args):
-    is_application = build_prepare(args.verbose)
-    build_app(args.verbose, args.jobs, is_application)
+    is_application = build_prepare(args.verbose, args.optimize)
+    build_app(args.debug, args.verbose, args.jobs, is_application)
 
 def run_app(args, verbose):
     if verbose:
@@ -634,8 +649,8 @@ def style_source(code):
                      Terminal256Formatter(style='monokai')).rstrip()
 
 def do_run(_parser, args):
-    if build_prepare(args.verbose):
-        build_app(args.verbose, args.jobs, True)
+    if build_prepare(args.verbose, args.optimize):
+        build_app(args.debug, args.verbose, args.jobs, True)
         run_app(args.args, args.verbose)
     else:
         main_1 = style_source('def main():\n')
@@ -648,13 +663,18 @@ def do_run(_parser, args):
         print(f'│ {main_1}                                               │')
         print(f"│ {main_2}                                │")
         print('└───────────────────────────────────────────────────────────┘')
-        sys.exit(1)
+
+        raise Exception()
 
 def do_test(_parser, args):
-    build_prepare(args.verbose)
+    build_prepare(args.verbose, args.optimize)
     command = [
         'make', '-f', 'build/Makefile', '-j', str(args.jobs), 'test', 'TEST=yes'
     ]
+
+    if args.debug:
+        command += ['TRANSPILE_DEBUG=--debug']
+
     run(command, 'Building tests', args.verbose)
     run(['./build/test'], 'Running tests', args.verbose)
 
@@ -701,34 +721,42 @@ def do_lint(_parser, args):
         print_lint_message(item)
 
     if returncode != 0:
-        sys.exit(1)
+        raise Exception()
 
 def do_transpile(_parser, args):
-    module_hpp = os.path.join(args.package_name, args.mysfile + '.hpp')
-    hpp_path = os.path.join(args.outdir, 'include', module_hpp)
-    cpp_path = os.path.join(args.outdir,
-                            'src',
-                            args.package_name,
-                            args.mysfile + '.cpp')
-    mys_path = os.path.join(args.package_path, 'src', args.mysfile)
+    sources = []
 
-    with open(mys_path) as fin:
-        try:
-            header, source = transpile(fin.read(),
-                                       mys_path,
-                                       module_hpp,
-                                       args.skip_tests)
-        except Exception as e:
-            sys.exit(str(e))
+    for i, mysfile in enumerate(args.mysfiles):
+        mys_path = os.path.join(args.package_path[i], 'src', mysfile)
+        module_hpp = os.path.join(args.package_name[i], mysfile + '.hpp')
+        module = '.'.join(module_hpp[:-8].split('/'))
+        hpp_path = os.path.join(args.outdir, 'include', module_hpp)
+        cpp_path = os.path.join(args.outdir,
+                                'src',
+                                args.package_name[i],
+                                mysfile + '.cpp')
 
-    os.makedirs(os.path.dirname(hpp_path), exist_ok=True)
-    os.makedirs(os.path.dirname(cpp_path), exist_ok=True)
+        with open(mys_path, 'r') as fin:
+            sources.append(Source(fin.read(),
+                                  mysfile,
+                                  module,
+                                  mys_path,
+                                  module_hpp,
+                                  args.skip_tests[i] == 'yes',
+                                  hpp_path,
+                                  cpp_path))
 
-    with open (hpp_path, 'w') as fout:
-        fout.write(header)
+    generated = transpile(sources)
 
-    with open (cpp_path, 'w') as fout:
-        fout.write(source)
+    for source, (hpp_code, cpp_code) in zip(sources, generated):
+        os.makedirs(os.path.dirname(source.hpp_path), exist_ok=True)
+        os.makedirs(os.path.dirname(source.cpp_path), exist_ok=True)
+
+        with open (source.hpp_path, 'w') as fout:
+            fout.write(hpp_code)
+
+        with open (source.cpp_path, 'w') as fout:
+            fout.write(cpp_code)
 
 def publish_create_release_package(config, verbose, archive):
     with open('setup.py', 'w') as fout:
@@ -798,13 +826,13 @@ def do_publish(_parser, args):
         os.chdir(path)
 
 def do_style(_parser, _args):
-    config = read_package_configuration()
+    read_package_configuration()
 
     print(f'┌───────────────────────────────────────────────────────── {INFO}  ─┐')
     print('│ This subcommand is not yet implemented.                      │')
     print('└──────────────────────────────────────────────────────────────┘')
 
-    sys.exit(1)
+    raise Exception()
 
 def do_help(parser, _args):
     parser.print_help()
@@ -823,11 +851,31 @@ Available subcommands are:
     {cyan('publish')}  Publish a release.
 '''
 
+def add_verbose_argument(subparser):
+    subparser.add_argument('-v', '--verbose',
+                           action='store_true',
+                           help='Verbose output.')
+
+def add_jobs_argument(subparser):
+    subparser.add_argument(
+        '-j', '--jobs',
+        type=int,
+        default=default_jobs(),
+        help='Maximum number of parallel jobs (default: %(default)s).')
+
+def add_optimize_argument(subparser, default):
+    subparser.add_argument(
+        '-o', '--optimize',
+        default=default,
+        choices=['speed', 'size', 'debug'],
+        help='Optimize the build for given level (default: %(default)s).')
+
 def main():
     parser = argparse.ArgumentParser(
         description=DESCRIPTION,
         formatter_class=argparse.RawDescriptionHelpFormatter)
 
+    parser.add_argument('-d', '--debug', action='store_true')
     parser.add_argument('--version',
                         action='version',
                         version=__version__,
@@ -854,26 +902,18 @@ def main():
     subparser = subparsers.add_parser(
         'build',
         description='Build the appliaction.')
-    subparser.add_argument('-v', '--verbose',
-                           action='store_true',
-                           help='Verbose output.')
-    subparser.add_argument('-j', '--jobs',
-                           type=int,
-                           default=default_jobs(),
-                           help='Maximum number of parallel jobs (default: %(default)s).')
+    add_verbose_argument(subparser)
+    add_jobs_argument(subparser)
+    add_optimize_argument(subparser, 'speed')
     subparser.set_defaults(func=do_build)
 
     # The run subparser.
     subparser = subparsers.add_parser(
         'run',
         description='Build and run the application.')
-    subparser.add_argument('-v', '--verbose',
-                           action='store_true',
-                           help='Verbose output.')
-    subparser.add_argument('-j', '--jobs',
-                           type=int,
-                           default=default_jobs(),
-                           help='Maximum number of parallel jobs (default: %(default)s).')
+    add_verbose_argument(subparser)
+    add_jobs_argument(subparser)
+    add_optimize_argument(subparser, 'speed')
     subparser.add_argument('args', nargs='*')
     subparser.set_defaults(func=do_run)
 
@@ -881,13 +921,9 @@ def main():
     subparser = subparsers.add_parser(
         'test',
         description='Build and run tests.')
-    subparser.add_argument('-v', '--verbose',
-                           action='store_true',
-                           help='Verbose output.')
-    subparser.add_argument('-j', '--jobs',
-                           type=int,
-                           default=default_jobs(),
-                           help='Maximum number of parallel jobs (default: %(default)s).')
+    add_verbose_argument(subparser)
+    add_jobs_argument(subparser)
+    add_optimize_argument(subparser, 'debug')
     subparser.set_defaults(func=do_test)
 
     # The clean subparser.
@@ -900,38 +936,36 @@ def main():
     subparser = subparsers.add_parser(
         'lint',
         description='Perform static code analysis.')
-    subparser.add_argument('-j', '--jobs',
-                           type=int,
-                           default=default_jobs(),
-                           help='Maximum number of parallel jobs (default: %(default)s).')
+    add_jobs_argument(subparser)
     subparser.set_defaults(func=do_lint)
 
     # The transpile subparser.
     subparser = subparsers.add_parser(
         'transpile',
-        description='Transpile given Mys file to C++ header and source files.')
+        description='Transpile given Mys file(s) to C++ header and source files.')
     subparser.add_argument('-o', '--outdir',
                            default='.',
                            help='Output directory.')
     subparser.add_argument('-p', '--package-path',
                            required=True,
+                           action='append',
                            help='Package path.')
     subparser.add_argument('-n', '--package-name',
                            required=True,
+                           action='append',
                            help='Package name.')
     subparser.add_argument('-s', '--skip-tests',
-                           action='store_true',
+                           action='append',
+                           choices=['yes', 'no'],
                            help='Skip tests.')
-    subparser.add_argument('mysfile')
+    subparser.add_argument('mysfiles', nargs='+')
     subparser.set_defaults(func=do_transpile)
 
     # The publish subparser.
     subparser = subparsers.add_parser(
         'publish',
         description='Publish a release.')
-    subparser.add_argument('-v', '--verbose',
-                           action='store_true',
-                           help='Verbose output.')
+    add_verbose_argument(subparser)
     subparser.add_argument('-u', '--username',
                            help='Registry username.')
     subparser.add_argument('-p', '--password',
@@ -961,6 +995,9 @@ def main():
     try:
         args.func(parser, args)
     except Exception as e:
+        if args.debug:
+            print_exc()
+
         sys.exit(str(e))
     except KeyboardInterrupt:
         print()
