@@ -2,13 +2,13 @@
 #include "std/timer.hpp"
 #include "std/thread.hpp"
 
-List<String> create_args(int argc, const char *argv[])
+std::shared_ptr<List<String>> create_args(int argc, const char *argv[])
 {
     int i;
-    List<String> args({});
+    auto args = std::make_shared<List<String>>();
 
     for (i = 0; i < argc; i++) {
-        args.append(argv[i]);
+        args->append(argv[i]);
     }
 
     return args;
@@ -108,14 +108,15 @@ int main()
 
 #elif defined(MYS_APPLICATION)
 
-extern int package_main(int argc, const char *argv[]);
+extern void package_main(int argc, const char *argv[]);
 
 int main(int argc, const char *argv[])
 {
     int res;
 
     try {
-        res = package_main(argc, argv);
+        package_main(argc, argv);
+        res = 0;
     } catch (std::exception &e) {
         std::cerr << e << std::endl;
         res = 1;
