@@ -1,21 +1,11 @@
 #pragma once
 
 #include <iostream>
-#include <cstdint>
-#include <cmath>
-#include <tuple>
-#include <utility>
 #include <vector>
-#include <string>
-#include <iterator>
-#include <algorithm>
 #include <memory>
 #include <unordered_map>
-#include <cassert>
-#include <functional>
-#include "iter.hpp"
 #include <sstream>
-#include <optional>
+#include <cstring>
 
 typedef int8_t i8;
 typedef int16_t i16;
@@ -95,7 +85,7 @@ public:
     {
     }
 
-    void operator+=(const String& other)
+    void operator+=(const String& other) const
     {
         *m_string += *other.m_string;
     }
@@ -293,34 +283,34 @@ template<typename T>
 class List
 {
 public:
-    std::shared_ptr<std::vector<T>> m_list;
+    std::vector<T> m_list;
 
-    List() : m_list(std::make_shared<std::vector<T>>())
+    List() : m_list(std::vector<T>())
     {
     }
 
-    List(std::initializer_list<T> il) : m_list(std::make_shared<std::vector<T>>(il))
+    List(std::initializer_list<T> il) : m_list(std::vector<T>(il))
     {
     }
 
-    T operator[](size_t pos) const
+    T get(size_t pos) const
     {
-        return m_list->at(pos);
+        return m_list.at(pos);
     }
 
-    T& operator[](size_t pos)
+    T& get(size_t pos)
     {
-        return m_list->at(pos);
+        return m_list.at(pos);
     }
 
-    List<T> operator*(int value) const
+    std::shared_ptr<List<T>> operator*(int value) const
     {
-        List<T> res;
+        auto res = std::make_shared<List<T>>();
         int i;
 
         for (i = 0; i < value; i++) {
-            for (auto item: (*m_list)) {
-                res.append(item);
+            for (auto item: m_list) {
+                res->append(item);
             }
         }
 
@@ -331,12 +321,12 @@ public:
     {
         size_t i;
 
-        if (m_list->size() != other.m_list->size()) {
+        if (m_list.size() != other.m_list.size()) {
             return false;
         }
 
-        for (i = 0; i < m_list->size(); i++) {
-            if ((*m_list)[i] != (*other.m_list)[i]) {
+        for (i = 0; i < m_list.size(); i++) {
+            if (m_list[i] != other.m_list[i]) {
                 return false;
             }
         }
@@ -346,22 +336,42 @@ public:
 
     typename std::vector<T>::iterator begin() const
     {
-        return m_list->begin();
+        return m_list.begin();
     }
 
     typename std::vector<T>::iterator end() const
     {
-        return m_list->end();
+        return m_list.end();
+    }
+
+    typename std::vector<T>::iterator begin()
+    {
+        return m_list.begin();
+    }
+
+    typename std::vector<T>::iterator end()
+    {
+        return m_list.end();
     }
 
     typename std::vector<T>::reverse_iterator rbegin() const
     {
-        return m_list->rbegin();
+        return m_list.rbegin();
     }
 
     typename std::vector<T>::reverse_iterator rend() const
     {
-        return m_list->rend();
+        return m_list.rend();
+    }
+
+    typename std::vector<T>::reverse_iterator rbegin()
+    {
+        return m_list.rbegin();
+    }
+
+    typename std::vector<T>::reverse_iterator rend()
+    {
+        return m_list.rend();
     }
 
     bool operator!=(const List<T>& other) const
@@ -370,26 +380,26 @@ public:
     }
 
     void append(const T& item)
-    { 
-        m_list->push_back(item);
+    {
+        m_list.push_back(item);
     }
 
     int __len__() const
     {
-        return m_list->size();
+        return m_list.size();
     }
 
     T __min__() const
     {
         T minimum;
 
-        if (m_list->size() == 0) {
+        if (m_list.size() == 0) {
             throw ValueError("min() arg is an empty sequence");
         }
 
-        minimum = (*m_list)[0];
+        minimum = m_list[0];
 
-        for (auto item: *m_list) {
+        for (auto item: m_list) {
             if (item < minimum) {
                 minimum = item;
             }
@@ -402,13 +412,13 @@ public:
     {
         T maximum;
 
-        if (m_list->size() == 0) {
+        if (m_list.size() == 0) {
             throw ValueError("max() arg is an empty sequence");
         }
 
-        maximum = (*m_list)[0];
+        maximum = m_list[0];
 
-        for (auto item: *m_list) {
+        for (auto item: m_list) {
             if (item > maximum) {
                 maximum = item;
             }
@@ -421,7 +431,7 @@ public:
     {
         T sum = 0;
 
-        for (auto item: *m_list) {
+        for (auto item: m_list) {
             sum += item;
         }
 
@@ -430,7 +440,7 @@ public:
 
     bool __contains__(const T& value) const
     {
-        for (auto item: *m_list) {
+        for (auto item: m_list) {
             if (item == value) {
                 return true;
             }
@@ -441,19 +451,23 @@ public:
 };
 
 template <typename T> std::ostream&
-operator<<(std::ostream& os, const List<std::shared_ptr<T>>& vec)
+operator<<(std::ostream& os, const std::shared_ptr<List<T>>& vec)
 {
     const char *delim_p;
 
-    os << "[";
-    delim_p = "";
+    if (vec == nullptr) {
+        os << "None";
+    } else {
+        os << "[";
+        delim_p = "";
 
-    for (auto item = vec.begin(); item != vec.end(); item++, delim_p = ", ") {
-        os << delim_p << **item;
+        for (auto item = vec->begin(); item != vec->end(); item++, delim_p = ", ") {
+            os << delim_p << *item;
+        }
+
+        os << "]";
     }
 
-    os << "]";
-
     return os;
 }
 
@@ -484,17 +498,28 @@ operator<<(std::ostream& os, const List<T>& obj)
 }
 
 // Dicts.
-template<typename TK, typename TV> using Dict = std::shared_ptr<std::unordered_map<TK, TV>>;
+template<typename TK, typename TV>
+class Dict
+{
+public:
+    std::unordered_map<TK, TV> m_map;
+
+    Dict() : m_map(std::unordered_map<TK, TV>())
+    {
+    }
+};
 
 template<class TK, class TV> std::ostream&
-operator<<(std::ostream& os, const std::unordered_map<TK, TV>& map)
+operator<<(std::ostream& os, const std::shared_ptr<Dict<TK, TV>>& dict)
 {
     const char *delim_p;
 
     os << "{";
     delim_p = "";
 
-    for (auto item = map.begin(); item != map.end(); item++, delim_p = ", ") {
+    for (auto item = dict->m_map.begin();
+         item != dict->m_map.end();
+         item++, delim_p = ", ") {
         os << delim_p << item->first << ": " << item->second;
     }
 
@@ -503,12 +528,6 @@ operator<<(std::ostream& os, const std::unordered_map<TK, TV>& map)
     return os;
 }
 
-template<class TK, class TV> Dict<TK, TV>
-MakeDict(std::initializer_list<typename std::unordered_map<TK, TV>::value_type> il)
-{
-    return std::make_shared<std::unordered_map<TK, TV>>(il);
-}
-
 // Integer power (a ** b).
 template <typename TB, typename TE>
 TB ipow(TB base, TE exp)
@@ -535,49 +554,9 @@ std::ostream& operator<<(std::ostream& os, const std::exception& e);
         throw AssertionError(#cond);            \
     }
 
-template <typename T>
-auto len(T obj)
-{
-    if constexpr (std::is_class<T>::value) {
-        return obj.__len__();
-    } else {
-        if constexpr (std::is_same<T, int>::value) {
-            static_assert(!sizeof(T *), "Object of type 'int' has no len().");
-        } else if constexpr (std::is_same<T, float>::value) {
-            static_assert(!sizeof(T *), "Object of type 'float' has no len().");
-        } else {
-            static_assert(!sizeof(T *), "Object of unknown type has no len().");
-        }
-
-        return 0;
-    }
-}
-
 std::ostream&
 operator<<(std::ostream& os, const String& obj);
 
-template <typename T>
-auto str(T obj)
-{
-    if constexpr (std::is_class<T>::value) {
-        return obj.__str__();
-    } else {
-        String res(obj);
-
-        return res;
-    }
-}
-
-template <typename T>
-auto to_int(T obj)
-{
-    if constexpr (std::is_class<T>::value) {
-        return obj.__int__();
-    } else {
-        return (int)obj;
-    }
-}
-
 static inline String operator*(int value, const String& string)
 {
     return string * value;
@@ -598,7 +577,7 @@ static inline List<T> operator*(int value, const List<T>& list)
     return list * value;
 }
 
-List<String> create_args(int argc, const char *argv[]);
+std::shared_ptr<List<String>> create_args(int argc, const char *argv[]);
 
 // A text file.
 class TextIO {
@@ -674,7 +653,7 @@ template <typename T, typename... Tail>
 auto min(T&& obj, Tail&&... tail)
 {
     if constexpr (sizeof...(tail) == 0) {
-        return obj.__min__();
+        return obj->__min__();
     } else {
         return vmin(obj, tail...);
     }
@@ -694,7 +673,7 @@ template <typename T, typename... Tail>
 auto max(T&& obj, Tail&&... tail)
 {
     if constexpr (sizeof...(tail) == 0) {
-        return obj.__max__();
+        return obj->__max__();
     } else {
         return vmax(obj, tail...);
     }
@@ -709,7 +688,7 @@ auto sum(T obj)
 template <typename TI, typename TC>
 bool contains(const TI& item, const TC& container)
 {
-    return container.__contains__(item);
+    return container->__contains__(item);
 }
 
 using std::abs;
@@ -738,7 +717,7 @@ public:
         String res;
         ssize_t i;
 
-        for (i = 0; i < size && m_pos < len(m_string); i++) {
+        for (i = 0; i < size && m_pos < m_string.__len__(); i++) {
             char a[2] = {(*m_string.m_string)[m_pos], '\0'};
             res += a;
             m_pos++;
@@ -769,66 +748,6 @@ static inline String chr(int value)
         throw AssertionError("assert_ne failed");                       \
     }
 
-#define assert_gt(v1, v2)                                               \
-    if (!((v1) > (v2))) {                                               \
-        std::cout << "Assert: " << (v1) << " <= " << (v2) << std::endl; \
-                                                                        \
-        throw AssertionError("assert_gt failed");                       \
-    }
-
-#define assert_ge(v1, v2)                                               \
-    if (!((v1) >= (v2))) {                                              \
-        std::cout << "Assert: " << (v1) << " < " << (v2) << std::endl;  \
-                                                                        \
-        throw AssertionError("assert_ge failed");                       \
-    }
-
-#define assert_lt(v1, v2)                                               \
-    if (!((v1) < (v2))) {                                               \
-        std::cout << "Assert: " << (v1) << " >= " << (v2) << std::endl; \
-                                                                        \
-        throw AssertionError("assert_lt failed");                       \
-    }
-
-#define assert_le(v1, v2)                                               \
-    if (!((v1) <= (v2))) {                                              \
-        std::cout << "Assert: " << (v1) << " > " << (v2) << std::endl;  \
-                                                                        \
-        throw AssertionError("assert_le failed");                       \
-    }
-
-#define assert_in(item, iterable)                               \
-    if (!contains((item), (iterable))) {                        \
-        std::cout                                               \
-            << "Assert: " << (item) << " not in " << (iterable) \
-            << std::endl;                                       \
-                                                                \
-        throw AssertionError("assert_in failed");               \
-    }
-
-#define assert_not_in(item, iterable)                           \
-    if (contains((item), (iterable))) {                         \
-        std::cout                                               \
-            << "Assert: " << (item) << " in " << (iterable)     \
-            << std::endl;                                       \
-                                                                \
-        throw AssertionError("assert_not_in failed");           \
-    }
-
-#define assert_true(value)                                      \
-    if (!(value)) {                                             \
-        std::cout << "Assert: " << (value) << std::endl;        \
-                                                                \
-        throw AssertionError("assert_true failed");             \
-    }
-
-#define assert_false(value)                                     \
-    if (value) {                                                \
-        std::cout << "Assert: " << (value) << std::endl;        \
-                                                                \
-        throw AssertionError("assert_false failed");            \
-    }
-
 class Test;
 
 extern Test *tests_p;
@@ -868,3 +787,289 @@ public:
     }
 
 };
+
+/* slice(), enumerate() and range() used in for loops. */
+
+struct Slice {
+    i64 m_begin;
+    i64 m_end;
+    i64 m_step;
+
+    Slice(i64 begin, i64 end, i64 step, i64 length) {
+        if (step == 0) {
+            throw ValueError("slice step can't be zero");
+        }
+
+        if (begin < 0) {
+            begin += length;
+
+            if (begin < 0) {
+                begin = 0;
+            }
+        }
+
+        if (end < 0) {
+            end += length;
+
+            if (end < 0) {
+                end = 0;
+            }
+        }
+
+        if (((begin < end) && (step < 0))
+            || ((begin > end) && (step > 0))
+            || (begin == end)) {
+            begin = 0;
+            end = 0;
+            step = 1;
+        }
+
+        m_begin = begin;
+        m_end = end;
+        m_step = step;
+    }
+
+    i64 length() {
+        if (m_begin <= m_end) {
+            return (m_end - m_begin + m_step - 1) / m_step;
+        } else {
+            return (m_end - m_begin + m_step + 1) / m_step;
+        }
+    }
+};
+
+class OpenSlice {
+
+public:
+    i64 m_begin;
+
+    OpenSlice(i64 begin) {
+        if (begin < 0) {
+            begin = 0;
+        }
+
+        m_begin = begin;
+    }
+};
+
+template <typename T>
+class Range {
+
+public:
+    T m_begin;
+    T m_end;
+    T m_step;
+    T m_next;
+
+    Range(T end) : m_begin(0), m_end(end), m_step(1) {
+    }
+
+    Range(T begin, T end) : m_begin(begin), m_end(end), m_step(1) {
+    }
+
+    Range(T begin, T end, T step) : m_begin(begin), m_end(end), m_step(step) {
+    }
+
+    void iter() {
+        m_next = m_begin;
+    }
+
+    T next() {
+        T next = m_next;
+        m_next += m_step;
+        return next;
+    }
+
+    void slice(Slice& slice) {
+        T begin = (m_begin + slice.m_begin * m_step);
+
+        if (begin > m_end) {
+            begin = m_end;
+        }
+
+        m_begin = begin;
+        T end = m_begin + slice.length() * slice.m_step;
+
+        if (end > m_end) {
+            end = m_end;
+        }
+
+        m_end = end;
+        m_step *= slice.m_step;
+    }
+
+    void slice(OpenSlice& slice) {
+        m_begin += (slice.m_begin * m_step);
+    }
+
+    void reversed() {
+        T begin;
+        T l;
+
+        begin = m_begin;
+        l = length();
+        m_begin = begin + (l - 1) * m_step;
+        m_end = m_begin - (l - 1) * m_step;
+
+        if (m_step > 0) {
+            m_end--;
+        } else {
+            m_end++;
+        }
+
+        m_step *= -1;
+    }
+
+    i64 length() {
+        if (m_step > 0) {
+            return (m_end - m_begin + m_step - 1) / m_step;
+        } else {
+            return (m_end - m_begin + m_step + 1) / m_step;
+        }
+    }
+};
+
+template <typename T>
+class Enumerate {
+
+public:
+    T m_begin;
+    T m_end;
+    T m_step;
+    T m_next;
+
+    Enumerate(T begin, T end) {
+        m_begin = begin;
+        m_end = begin + end;
+        m_step = 1;
+    }
+
+    void iter() {
+        m_next = m_begin;
+    }
+
+    T next() {
+        T next = m_next;
+        m_next += m_step;
+        return next;
+    }
+
+    void slice(Slice& slice) {
+        T begin = (m_begin + slice.m_begin * m_step);
+
+        if (begin > m_end) {
+            begin = m_end;
+        }
+
+        m_begin = begin;
+        T end = m_begin + slice.length() * slice.m_step;
+
+        if (end > m_end) {
+            end = m_end;
+        }
+
+        m_end = end;
+        m_step *= slice.m_step;
+    }
+
+    void slice(OpenSlice& slice) {
+        m_begin += slice.m_begin;
+    }
+
+    void reversed() {
+        T begin;
+        T l;
+
+        begin = m_begin;
+        l = length();
+        m_begin = begin + (l - 1) * m_step;
+        m_end = m_begin - (l - 1) * m_step;
+
+        if (m_step > 0) {
+            m_end--;
+        } else {
+            m_end++;
+        }
+
+        m_step *= -1;
+    }
+
+    i64 length() {
+        if (m_step > 0) {
+            return (m_end - m_begin + m_step - 1) / m_step;
+        } else {
+            return (m_end - m_begin + m_step + 1) / m_step;
+        }
+    }
+};
+
+template <typename T>
+class Data {
+
+public:
+    T m_begin;
+    T m_end;
+    T m_step;
+    T m_next;
+
+    Data(T end) : m_begin(0), m_end(end), m_step(1) {
+    }
+
+    void iter() {
+        m_next = m_begin;
+    }
+
+    T next() {
+        T next = m_next;
+        m_next += m_step;
+        return next;
+    }
+
+    void slice(Slice& slice) {
+        T begin = (m_begin + slice.m_begin * m_step);
+
+        if (begin > m_end) {
+            begin = m_end;
+        }
+
+        m_begin = begin;
+        T end = m_begin + slice.length() * slice.m_step;
+
+        if (end > m_end) {
+            end = m_end;
+        }
+
+        m_end = end;
+        m_step *= slice.m_step;
+    }
+
+    void slice(OpenSlice& slice) {
+        m_begin += slice.m_begin;
+    }
+
+    void reversed() {
+        T begin;
+        T l;
+
+        begin = m_begin;
+        l = length();
+        m_begin = begin + (l - 1) * m_step;
+        m_end = m_begin - (l - 1) * m_step;
+
+        if (m_step > 0) {
+            m_end--;
+        } else {
+            m_end++;
+        }
+
+        m_step *= -1;
+    }
+
+    i64 length() {
+        if (m_step > 0) {
+            return (m_end - m_begin + m_step - 1) / m_step;
+        } else {
+            return (m_end - m_begin + m_step + 1) / m_step;
+        }
+    }
+};
