@@ -254,9 +254,6 @@ class MysTest(unittest.TestCase):
                 with patch('subprocess.run', run_mock):
                     mys.cli.main()
 
-            if sys.version_info < (3, 8):
-                return
-
             # sdist.
             call = run_mock.call_args_list[0]
             self.assertEqual(call.args[0][1:], ['setup.py', 'sdist'])
@@ -339,15 +336,13 @@ class MysTest(unittest.TestCase):
             # Add dependencies.
             with open('package.toml', 'a') as fout:
                 fout.write('bar = "0.3.0"\n')
-                fout.write('math = "*"\n')
 
             # Import from bar.
             with open('src/main.mys', 'w') as fout:
                 print('from bar import hello', file=fout)
-                print('from math import pi', file=fout)
                 print('', file=fout)
                 print('def main():', file=fout)
-                print('    v: string = str(pi)', file=fout)
+                print('    v = "3.14"', file=fout)
                 print('    hello(v)', file=fout)
 
             # Run.
@@ -500,9 +495,9 @@ class MysTest(unittest.TestCase):
 
         module_names = [
             'calc',
+            'enums',
             'hello_world',
             'loops',
-            'strings',
             'various'
         ]
 
@@ -516,12 +511,12 @@ class MysTest(unittest.TestCase):
 
         try:
             # Test.
-            with patch('sys.argv', ['mys', 'test', '--verbose']):
+            with patch('sys.argv', ['mys', '--debug', 'test', '--verbose']):
                 mys.cli.main()
 
-            # Lint.
-            with patch('sys.argv', ['mys', 'lint']):
-                mys.cli.main()
+            # # Lint.
+            # with patch('sys.argv', ['mys', 'lint']):
+            #     mys.cli.main()
         finally:
             os.chdir(path)
 
